"""Extended Textual OptionList widget."""

from typing import Any

from textual import widgets
from textual.widgets._option_list import NewOptionListContent

from .._extensions import WidgetExtension


class OptionList(WidgetExtension, widgets.OptionList):
    """An extended vertical option list with bounce-bar highlighting."""

    def __init__(
        self,
        *content: NewOptionListContent,
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
        disabled: bool = False,
        **extension_configs: Any,
    ) -> None:
        """Initialize the option list.

        Args:
            *content: The content for the option list.
            name: The name of the option list.
            id: The ID of the option list in the DOM.
            classes: The CSS classes of the option list.
            disabled: Whether the option list is disabled or not.
            extension_configs: Widget extension configurations, such as dynamically provided local callbacks by name.
        """
        super().__init__(
            *content,
            name=name,
            id=id,
            classes=classes,
            disabled=disabled,
        )
        self.__extend_widget__(**extension_configs)



"""Extended Textual Input widget."""

from typing import Any
from typing import Iterable

from rich.highlighter import Highlighter
from textual import widgets
from textual.suggester import Suggester
from textual.validation import Validator

from .._extensions import WidgetExtension


class TextInput(WidgetExtension, widgets.Input):
    """An extended text input widget.

    Renamed from "Input" to avoid conflicts with callback dependency types.
    """

    def __init__(
        self,
        value: str | None = None,
        placeholder: str = "",
        highlighter: Highlighter | None = None,
        password: bool = False,
        *,
        suggester: Suggester | None = None,
        validators: Validator | Iterable[Validator] | None = None,
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
        disabled: bool = False,
        **extension_configs: Any,
    ) -> None:
        """Initialize the text input widget.

        Args:
            value: An optional default value for the input.
            placeholder: Optional placeholder text for the input.
            highlighter: An optional highlighter for the input.
            password: Flag to say if the field should obfuscate its content.
            suggester: Suggester associated with this input instance.
            validators: An iterable of validators that the Input value will be checked against.
            name: Optional name for the input widget.
            id: Optional ID for the widget.
            classes: Optional initial classes for the widget.
            disabled: Whether the input is disabled or not.
            extension_configs: Widget extension configurations, such as dynamically provided local callbacks by name.
        """
        super().__init__(
            value=value,
            placeholder=placeholder,
            highlighter=highlighter,
            password=password,
            suggester=suggester,
            validators=validators,
            name=name,
            id=id,
            classes=classes,
            disabled=disabled,
        )
        self.__extend_widget__(**extension_configs)



"""Extended Textual Tab Content widgets."""

from typing import Any

from rich.text import TextType
from textual import widgets
from textual.widget import Widget

from .._extensions import WidgetExtension


class TabbedContent(WidgetExtension, widgets.TabbedContent):
    """An extended container with associated tabs to toggle content visibility."""

    def __init__(
        self,
        *titles: TextType,
        initial: str = "",
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
        disabled: bool = False,
        **extension_configs: Any,
    ) -> None:
        """Initialize a TabbedContent widgets.

        Args:
            *titles: Positional argument will be used as title.
            initial: The id of the initial tab, or empty string to select the first tab.
            name: The name of the button.
            id: The ID of the button in the DOM.
            classes: The CSS classes of the button.
            disabled: Whether the button is disabled or not.
            extension_configs: Widget extension configurations, such as dynamically provided local callbacks by name.
        """
        super().__init__(
            *titles,
            initial=initial,
            name=name,
            id=id,
            classes=classes,
            disabled=disabled,
        )
        self.__extend_widget__(**extension_configs)


class TabPane(WidgetExtension, widgets.TabPane):
    """An extended container for switchable content, with additional title."""

    def __init__(
        self,
        title: TextType,
        *children: Widget,
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
        disabled: bool = False,
        **extension_configs: Any,
    ) -> None:
        """Initialize a TabPane.

        Args:
            title: Title of the TabPane (will be displayed in a tab label).
            *children: Widget to go inside the TabPane.
            name: Optional name for the TabPane.
            id: Optional ID for the TabPane.
            classes: Optional initial classes for the widget.
            disabled: Whether the TabPane is disabled or not.
            extension_configs: Widget extension configurations, such as dynamically provided local callbacks by name.
        """
        super().__init__(
            title,
            *children,
            name=name,
            id=id,
            classes=classes,
            disabled=disabled,
        )
        self.__extend_widget__(**extension_configs)



"""Extended Textual Pretty widget."""

from typing import Any

from textual import widgets

from .._extensions import WidgetExtension


class Pretty(WidgetExtension, widgets.Pretty):
    """An extended pretty-printing widget."""

    DEFAULT_CSS = """
    Pretty {
        height: auto;
    }
    """

    def __init__(
        self,
        obj: Any,
        *,
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
        **extension_configs: Any,
    ) -> None:
        """Initialize the Pretty widget.

        Args:
            obj: The object to pretty-print.
            name: The name of the pretty widget.
            id: The ID of the pretty in the DOM.
            classes: The CSS classes of the pretty.
            extension_configs: Widget extension configurations, such as dynamically provided local callbacks by name.
        """
        super().__init__(
            object=obj,
            name=name,
            id=id,
            classes=classes,
        )
        self.__extend_widget__(**extension_configs)



"""Extended Textual Footer widget."""

from typing import Any

from textual import widgets

from .._extensions import WidgetExtension


class Footer(WidgetExtension, widgets.Footer):
    """A extended simple footer widget which docks itself to the bottom of the parent container."""

    def __init__(
        self,
        **extension_configs: Any,
    ) -> None:
        """Initialize the footer widget.

        Args:
            extension_configs: Widget extension configurations, such as dynamically provided local callbacks by name.
        """
        super().__init__()
        self.__extend_widget__(**extension_configs)



"""Extended Textual Markdown widgets."""

from typing import Any
from typing import Callable

from textual import widgets
from textual.widgets._markdown import MarkdownIt

from .._extensions import WidgetExtension


class Markdown(WidgetExtension, widgets.Markdown):
    """An extended widget for rendering markdown."""

    def __init__(
        self,
        markdown: str | None = None,
        *,
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
        parser_factory: Callable[[], MarkdownIt] | None = None,
        **extension_configs: Any,
    ) -> None:
        """Initialize a Markdown widget.

        Args:
            markdown: String containing Markdown or None to leave blank for now.
            name: The name of the widget.
            id: The ID of the widget in the DOM.
            classes: The CSS classes of the widget.
            parser_factory: A factory function to return a configured MarkdownIt instance.
                If "None`, a "gfm-like" parser is used.
            extension_configs: Widget extension configurations, such as dynamically provided local callbacks by name.
        """
        super().__init__(
            markdown=markdown,
            name=name,
            id=id,
            classes=classes,
            parser_factory=parser_factory,
        )
        self.__extend_widget__(**extension_configs)


class MarkdownViewer(WidgetExtension, widgets.MarkdownViewer):
    """An extended Markdown viewer widget."""

    def __init__(
        self,
        markdown: str | None = None,
        *,
        show_table_of_contents: bool = True,
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
        parser_factory: Callable[[], MarkdownIt] | None = None,
        **extension_configs: Any,
    ) -> None:
        """Initialize a Markdown Viewer widget.

        Args:
            markdown: String containing Markdown, or None to leave blank.
            show_table_of_contents: Show a table of contents in a sidebar.
            name: The name of the widget.
            id: The ID of the widget in the DOM.
            classes: The CSS classes of the widget.
            parser_factory: A factory function to return a configured MarkdownIt instance.
                If "None", a "gfm-like" parser is used.
            extension_configs: Widget extension configurations, such as dynamically provided local callbacks by name.
        """
        super().__init__(
            markdown=markdown,
            show_table_of_contents=show_table_of_contents,
            name=name,
            id=id,
            classes=classes,
            parser_factory=parser_factory,
        )
        self.__extend_widget__(**extension_configs)
